"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.useWindowScrollEffect = void 0;
var just_throttle_1 = __importDefault(require("just-throttle"));
var react_1 = require("react");
/**
 * Custom Hook that calls a callback function each time a scroll event occurs
 * @param effect callback function
 * @param options.waitingMs Waiting time after function execution
 * @see https://developer.mozilla.org/en-US/docs/Web/API/EventTarget/addEventListener#parameters
 */
var useWindowScrollEffect = function (effect, options) {
    if (options === void 0) { options = {}; }
    var scrollEffect = (0, react_1.useMemo)(function () {
        return (0, just_throttle_1.default)(function () {
            effect(window.scrollY);
        }, options.waitingMs || 0);
    }, [effect, options.waitingMs]);
    (0, react_1.useEffect)(function () {
        window.addEventListener('scroll', scrollEffect, options);
        return function () {
            window.removeEventListener('scroll', scrollEffect);
        };
    }, [options, scrollEffect]);
};
exports.useWindowScrollEffect = useWindowScrollEffect;
