"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.useWindowScrollInElement = void 0;
var react_1 = require("react");
var useRefValue_1 = require("./hooks/useRefValue");
var useWindowScrollEffect_1 = require("./hooks/useWindowScrollEffect");
var useWindowSize_1 = require("./hooks/useWindowSize");
var object_1 = require("./utils/object");
var DEFAULT_OPTIONS = {
    disableValueChangesOffscreen: false,
    scrollStartPosition: 'window-top',
    scrollEndPosition: 'window-top',
};
var FALLBACK_VALUES = {
    position: { top: 0, bottom: 0 },
    fraction: { top: 0, bottom: 0 },
    isInside: false,
};
/**
 * Hooks to get how much the window has scrolled from the target element.
 * @param targetElmRef target element
 * @param opts
 * @returns result
 * @returns result.position Get the position of the "top/bottom" of the window in px within the target element.
 * @returns result.position.top
 * @returns result.position.bottom
 * @returns result.fraction Get the position of the "top/bottom" of the window as a percentage of the target element's scrollable area with the height of the target element as 100%.
 * @returns result.fraction.top
 * @returns result.fraction.bottom
 * @returns result.isFraction window is within the scrollable area of the element * @returns result.isFraction window is within the scrollable area of the element?
 */
var useWindowScrollInElement = function (targetElmRef, opts) {
    if (opts === void 0) { opts = {}; }
    var options = (0, react_1.useMemo)(function () { return (__assign(__assign({}, DEFAULT_OPTIONS), opts)); }, [opts]);
    var currentTargetElm = (0, useRefValue_1.useRefValue)(targetElmRef);
    var _a = (0, react_1.useState)({
        top: null,
        bottom: null,
    }), position = _a[0], setPosition = _a[1];
    var windowHeight = (0, useWindowSize_1.useWindowSize)().height;
    var effect = (0, react_1.useCallback)(function () {
        var targetElm = targetElmRef.current;
        if (!targetElm) {
            return;
        }
        var positionTop = options.scrollStartPosition === 'window-top'
            ? targetElm.getBoundingClientRect().top * -1
            : options.scrollStartPosition === 'window-center'
                ? windowHeight / 2 - targetElm.getBoundingClientRect().top
                : windowHeight - targetElm.getBoundingClientRect().top;
        setPosition({ top: positionTop, bottom: positionTop + windowHeight });
    }, [options.scrollStartPosition, targetElmRef, windowHeight]);
    (0, react_1.useEffect)(effect, [effect]);
    (0, useWindowScrollEffect_1.useWindowScrollEffect)(effect, { waitingMs: options.waitingMs });
    var scrollableHeight = (0, react_1.useMemo)(function () {
        if (!currentTargetElm) {
            return NaN;
        }
        var scrollStartPosition = options.scrollStartPosition, scrollEndPosition = options.scrollEndPosition;
        switch (scrollStartPosition) {
            case 'window-top':
                switch (scrollEndPosition) {
                    case 'window-top':
                        return currentTargetElm.getBoundingClientRect().height;
                    case 'window-center':
                        return (currentTargetElm.getBoundingClientRect().height - windowHeight / 2);
                    case 'window-bottom':
                        return (currentTargetElm.getBoundingClientRect().height - windowHeight);
                }
            // eslint-disable-next-line no-fallthrough
            case 'window-center':
                switch (scrollEndPosition) {
                    case 'window-top':
                        return (currentTargetElm.getBoundingClientRect().height + windowHeight / 2);
                    case 'window-center':
                        return currentTargetElm.getBoundingClientRect().height;
                    case 'window-bottom':
                        return (currentTargetElm.getBoundingClientRect().height - windowHeight / 2);
                }
            // eslint-disable-next-line no-fallthrough
            case 'window-bottom':
                switch (scrollEndPosition) {
                    case 'window-top':
                        return (currentTargetElm.getBoundingClientRect().height + windowHeight);
                    case 'window-center':
                        return (currentTargetElm.getBoundingClientRect().height + windowHeight / 2);
                    case 'window-bottom':
                        return currentTargetElm.getBoundingClientRect().height;
                }
        }
    }, [currentTargetElm, options, windowHeight]);
    var isInside = (0, react_1.useMemo)(function () {
        return position.top !== null &&
            position.top + 1 > 0 &&
            position.top - 1 < scrollableHeight;
    }, [position.top, scrollableHeight]);
    if (options.disableValueChangesOffscreen && !isInside) {
        return FALLBACK_VALUES;
    }
    var _position = (0, object_1.mapValues)(position, function (v) { return v || 0; });
    return {
        position: _position,
        fraction: (0, object_1.mapValues)(_position, function (v) { return v / scrollableHeight || 0; }),
        isInside: isInside,
    };
};
exports.useWindowScrollInElement = useWindowScrollInElement;
